// http://www.cepstrum.co.jp/
//
// EDMA block read/write delay & talk through program for C6713 DSK
// mic in, line out, Fs=48kHz   
                      
#include "c6713dsk.h"             

#define UINT      unsigned int
#define USHORT    unsigned short

#define ALPHA     0.05
#define FRAME_LEN 256               // ping-pong buffer size (frame length)
#define BUF_LEN   96000

UINT    rxbuf0[FRAME_LEN];          // ping-pong buffer (input)
UINT    rxbuf1[FRAME_LEN];          // ping-pong buffer (input)
UINT    txbuf0[FRAME_LEN];          // ping-pong buffer (output)
UINT    txbuf1[FRAME_LEN];          // ping-pong buffer (output)
short   l_in[FRAME_LEN];
short   r_in[FRAME_LEN];
short   l_out[FRAME_LEN];
short   r_out[FRAME_LEN];
float   l_dc=0.0;
float   r_dc=0.0;

#pragma DATA_SECTION(buf, "sdram");      
short   buf[BUF_LEN];
int     ix_buf=0;

//================================================================    
// wait routine with LED flash

void wait(int loop) {
  int i, j;

  for (i=1; i<=8; i=i*2) {
    for (j=0; j<loop; j=j+1) *(char *)CPLD=i; 
  }
}  

//================================================================    
// SPI (McBSP0) transmit routine

void spi_tx(USHORT adrs, USHORT data) {
  USHORT tx_stream;
  
  while ((*(UINT *)McBSP0_SPCR&0x00020000)==0);  // wait until tx ready  
  tx_stream=(adrs&0x0f)<<9;
  tx_stream=tx_stream|(data&0x01ff);
  *(USHORT *)McBSP0_DXR=tx_stream;
}

//================================================================ 
// EDMA interrupt routine 

interrupt void edma_intr(void) {        
  short l, r;
  UINT  cipreg;
  UINT  temp;
  UINT  i;

  // block read A/D data
  cipreg=*(UINT *)CIPR;              // read interrupt pending reg
  if ((cipreg&0x01)!=0) {            // if TCC=0 (LINK0)
    *(UINT *)CIPR=0x01;              // clear interrupt pending reg.
    for (i=0; i<FRAME_LEN; i=i+1) {      
      l=rxbuf0[i]>>16;               // read rxbuf0[] --> l
      r=rxbuf0[i]&0xffff;            // read rxbuf0[] --> r    
      l_dc=(1.0-ALPHA)*l_dc+ALPHA*l; // accumulate DC level
      r_dc=(1.0-ALPHA)*r_dc+ALPHA*r; // accumulate DC level
      l_in[i]=l-l_dc;                // eliminate DC offset
      r_in[i]=r-r_dc;                // eliminate DC offset
    }
  } else if ((cipreg&0x02)!=0) {     // else if TCC=1 (LINK1)
    *(UINT *)CIPR=0x02;              // clear interrupt pending reg.
    for (i=0; i<FRAME_LEN; i=i+1) {
      l=rxbuf1[i]>>16;               // read rxbuf1[] --> l
      r=rxbuf1[i]&0xffff;            // read rxbuf1[] --> r     
      l_dc=(1.0-ALPHA)*l_dc+ALPHA*l; // accumulate DC level
      r_dc=(1.0-ALPHA)*r_dc+ALPHA*r; // accumulate DC level
      l_in[i]=l-l_dc;                // eliminate DC offset
      r_in[i]=r-r_dc;                // eliminate DC offset
    }
  }
  
  // something to do here
  for (i=0; i<FRAME_LEN; i=i+1) {
    l_out[i]=buf[ix_buf];            // read ring buffer
    buf[ix_buf]=l_in[i];             // write ring buffer
    ix_buf=(ix_buf+1)%BUF_LEN;       // update buffer pointer
    r_out[i]=l_in[i];                // pass through
  }
  
  // block write D/A data
  for (i=0; i<FRAME_LEN; i=i+1) {
    temp=l_out[i]<<16;                       
    temp=temp|(r_out[i]&0xffff);    
    if ((cipreg&0x01)!=0) {         // if TCC=0 (LINK0)
      txbuf0[i]=temp;               // write txbuf0[]
    } else if ((cipreg&0x02)!=0) {  // else if TCC=1 (LINK1)
      txbuf1[i]=temp;               // write txbuf1[]
    }
  }
  
}
            
//================================================================    

void main(void) {          
  UINT  temp;     
  UINT  dip_sw, led;
  
  CSR                 =0x00000100;  // disable all interrupts
  *(UINT *)CCFG       =0;           // cache off
  *(UINT *)DEVCFG     =0;           // ECLK<--SYSCLK3, McASP disable
 
  // PLL setup 
  *(UINT *)PLLCSR     =0x00000000;  // bypass PLL
  wait(1000);                       // wait
  *(UINT *)PLLCSR     =0x00000008;  // reset PLL
  *(UINT *)PLLDIV0    =0x00008000;  // PLLREF=50MHz/1=50MHz
  *(UINT *)PLLM       =0x00000009;  // PLLOUT=50MHz*9=450MHz
  *(UINT *)OSCDIV1    =0x00000004;  // CLKOUT3=50MHz/5=10MHz
  *(UINT *)PLLDIV3    =0x00008004;  // ECLKOUT=450MHz/5=90MHz
  wait(1000);                       // wait
  *(UINT *)PLLDIV2    =0x00008003;  // SYSCLK2=450MHz/4=112.5MHz
  wait(1000);                       // wait
  *(UINT *)PLLDIV1    =0x00008001;  // SYSCLK1=450MHz/2=225MHz  
  wait(1000);                       // wait
  *(UINT *)PLLCSR     =0x00000000;  // bring PLL out of reset
  wait(1000);                       // wait
  *(UINT *)PLLCSR     =0x00000001;  // enable PLL
 
  // EMIF setting       
  *(UINT *)EMIF_GBLCTL=0x00003078;  // EMIF global control
  *(UINT *)EMIF_CECTL0=0xffffff37;  // EMIF CE0 control, 32bit SDRAM   
  *(UINT *)EMIF_CECTL1=0xffffff07;  // EMIF CE1 control, 8bit async
  *(UINT *)EMIF_CECTL2=0xffffff27;  // EMIF CE2 control, 32bit async
  *(UINT *)EMIF_CECTL3=0xffffff27;  // EMIF CE3 control, 32bit async

  // EMIF SDRAM control setting for MT48LC2M32B2-6 (Micron)
  *(UINT *)EMIF_SDTIM =0x00000578;  // SDRAM refresh period          
  *(UINT *)EMIF_SDEXT =0x000a8529;  // SDRAM extension         
  *(UINT *)EMIF_SDCTL =0x47115000;  // SDRAM control                
  
  // daughter card reset  
  *(char *)(CPLD+1)   =0x08;        // daughter card reset on
  wait(10000);                      // wait
  *(char *)(CPLD+1)   =0;           // daughter card reset off

  // McBSP0 setting (CODEC control, DSP is master, SPI)
  *(UINT *)McBSP0_SPCR=0x00000000;  // reset McBSP0
  *(UINT *)McBSP0_SRGR=0x20000020;  // clock=112.5MHz/33=3.4MHz
  *(UINT *)McBSP0_PCR =0x00000a0a;  // use internal clock
  *(UINT *)McBSP0_XCR =0x00000040;  // 16bit/word
  *(UINT *)McBSP0_SPCR=0x00c01000;  // enable sample rate clock
  wait(1000);                       // wait
  *(UINT *)McBSP0_SPCR=0x00c11000;  // enable McBSP0 tx
  wait(1000);                       // wait
  
  // CODEC setting
  spi_tx( 9, 0x000);                // CODEC disable  
  spi_tx( 0, 0x080);                // disable line input (left)
  spi_tx( 1, 0x080);                // disable line input (right)
  spi_tx( 2, 0x0f7);                // enable headphone out (left)
  spi_tx( 3, 0x0f7);                // enable headphone out (right)
  spi_tx( 4, 0x015);                // mic line input (+20dB)
  spi_tx( 5, 0x000);                // enable HPF, disable de-emphasis
  spi_tx( 6, 0x000);                // all power on
  spi_tx( 7, 0x043);                // CODEC is master, DSP mode
  spi_tx( 8, 0x001);                // Fs=48kHz (USB mode, MCLK=12MHz)
  spi_tx( 9, 0x001);                // CODEC enable
  
  // McBSP1 setting (CODEC data, DSP is slave)
  *(UINT *)McBSP1_SPCR=0x00000000;  // reset McBSP1
  *(UINT *)McBSP1_PCR =0x00000000;  // use external clock
  *(UINT *)McBSP1_RCR =0x000100a0;  // 32bit/word, 1bit data delay
  *(UINT *)McBSP1_XCR =0x000000a0;  // 32bit/word
  wait(1000);                       // wait
  *(UINT *)McBSP1_SPCR=0x00010001;  // enable McBSP1   
  
  temp=*(UINT *)MUXL&~(0x1f<<21);   // read interrupt multiplexer (MUXL)
  temp=temp|(0x08<<21);             // set INTSEL8 <-- EDMA
  *(UINT *)MUXL=temp;               // update MUXL  
  ICR=0xffff;                       // clear all pending interrupt
  IER=0x00000102;                   // enable NMI & INT8
  CSR=CSR|1;                        // enable interrupt (global)
  
  *(UINT *)EER=0;                   // clear EDMA event enable reg.
  *(UINT *)ECR=0x0000ffff;          // clear all EDMA event
  temp=*(UINT *)ESEL3;              // read ESEL3
  *(UINT *)ESEL3=(temp&0x0000ffff)|0x0f0e0000;   // set EVTSEL14, 15
  *(UINT *)CIPR=0;                  // clear EDMA ch. interrupt pending reg.
  *(UINT *)CIER=3;                  // set EDMA ch. interrupt enable reg.
  
  // set EDMA EVENT15 parameter (McBSP1 RX) 
  *(UINT *)(EVENT15+OPT)=0x20310002;         // TCINT=1, TCC=1
  *(UINT *)(EVENT15+SRC)=McBSP1_DATA;   
  *(UINT *)(EVENT15+CNT)=FRAME_LEN;    
  *(UINT *)(EVENT15+DST)=(UINT)&rxbuf1;        
  *(UINT *)(EVENT15+IDX)=0;             
  *(UINT *)(EVENT15+RLD)=0x0000ffff&LINK0;  
  // set EDMA LINK0 parameter (McBSP1 RX)
  *(UINT *)(LINK0+OPT)  =0x20300002;         // TCINT=1, TCC=0
  *(UINT *)(LINK0+SRC)  =McBSP1_DATA;
  *(UINT *)(LINK0+CNT)  =FRAME_LEN;
  *(UINT *)(LINK0+DST)  =(UINT)&rxbuf0;
  *(UINT *)(LINK0+IDX)  =0;
  *(UINT *)(LINK0+RLD)  =0x0000ffff&LINK1;  
  // set EDMA LINK1 parameter (McBSP1 RX)
  *(UINT *)(LINK1+OPT)  =0x20310002;         // TCINT=1, TCC=1
  *(UINT *)(LINK1+SRC)  =McBSP1_DATA;
  *(UINT *)(LINK1+CNT)  =FRAME_LEN;
  *(UINT *)(LINK1+DST)  =(UINT)&rxbuf1;
  *(UINT *)(LINK1+IDX)  =0;
  *(UINT *)(LINK1+RLD)  =0x0000ffff&LINK0;   
  // set EDMA EVENT14 parameter (McBSP1 TX)
  *(UINT *)(EVENT14+OPT)=0x21000002;         // TCINT=0
  *(UINT *)(EVENT14+SRC)=(UINT)&txbuf1;   
  *(UINT *)(EVENT14+CNT)=FRAME_LEN;    
  *(UINT *)(EVENT14+DST)=McBSP1_DATA;        
  *(UINT *)(EVENT14+IDX)=0;             
  *(UINT *)(EVENT14+RLD)=0x0000ffff&LINK2;
  // set EDMA LINK2 parameter (McBSP1 TX)
  *(UINT *)(LINK2+OPT)  =0x21000002;         // TCINT=0
  *(UINT *)(LINK2+SRC)  =(UINT)&txbuf0;
  *(UINT *)(LINK2+CNT)  =FRAME_LEN;
  *(UINT *)(LINK2+DST)  =McBSP1_DATA;
  *(UINT *)(LINK2+IDX)  =0;
  *(UINT *)(LINK2+RLD)  =0x0000ffff&LINK3;
  // set EDMA LINK3 parameter (McBSP1 TX)
  *(UINT *)(LINK3+OPT)  =0x21000002;         // TCINT=0
  *(UINT *)(LINK3+SRC)  =(UINT)&txbuf1;
  *(UINT *)(LINK3+CNT)  =FRAME_LEN;
  *(UINT *)(LINK3+DST)  =McBSP1_DATA;
  *(UINT *)(LINK3+IDX)  =0;
  *(UINT *)(LINK3+RLD)  =0x0000ffff&LINK2;  
  
  // clear ping-pong buffer
  for (temp=0; temp<FRAME_LEN; temp=temp+1) {
    txbuf0[temp]=0; 
    txbuf1[temp]=0;
  }
  // clear ring buffer
  for (temp=0; temp<BUF_LEN; temp=temp+1) buf[temp]=0;    
     
  *(UINT *)EER          =0x0000c000;  // set EDMA event enable reg.
  *(UINT *)ESR          =0x0000c000;  // set EDMA event set reg.
  
  // forever loop
  while(1) {      
    dip_sw=~(*(char *)CPLD)>>4;       // read DIP SW
    led=dip_sw;                       // control LED by DIP SW
    *(char *)CPLD=led;                // LED on/off
  }           
  
}
       
